/* decode.h
   Header-Datei zu decode.c
   (c) Dirk Schwarzhans, Matthias Braun & c't 10/92 */

#define ROWS 24       /* Anzahl der Textzeilen und -spalten */
#define COLUMNS 40

typedef enum {YES = 1, NO = 0} BOOL;

/* Videotext Steuerzeichen */
typedef enum {ALPHABLACK = 0, ALPHARED = 1, ALPHAGREEN = 2,
              ALPHAYELLOW = 3, ALPHABLUE = 4, ALPHAMAGENTA = 5,
              ALPHACYAN = 6, ALPHAWHITE = 7, FLASH = 8, STEADY = 9,
              ENDBOX = 10, STARTBOX = 11, NORMAL = 12, DOUBLE = 13,
              SO = 14, SI = 15, GRAPHBLACK = 16, GRAPHRED = 17,
              GRAPHGREEN = 18, GRAPHYELLOW = 19, GRAPHBLUE = 20,
              GRAPHMAGENTA = 21, GRAPHCYAN = 22, GRAPHWHITE = 23,
              HIDE = 24, CONGRAPH = 25, SEPGRAPH = 26, ESC = 27,
              BLACKBGND = 28, NEWBGND = 29, HOLDGRAPH = 30,
              RELEASEGRAPH = 31} VTCtrl;

/* prft auf ein Steuerzeichen */   
#define ISCTRL(c) ((c) < 32)
/* berechnet die Farbe aus einen Videotext-Steuerzeichen */ 
#define CTRL2COLOR(a) ((a) & 7)
/* trennt die fr Videotext-Daten relevanten Bits ab */
#define VALIDBITS(a) ((a) & 0x7F)
/* prft auf ein Videotext-Grafikzeichen */
#define ISGRAPHICCHAR(a) ((a) & 0x20)

/* Videotext Farben */
typedef enum {VT_BLACK = 0, VT_RED = 1, VT_GREEN = 2, VT_YELLOW = 3,
        VT_BLUE = 4, VT_MAGENTA = 5, VT_CYAN = 6,
        VT_WHITE = 7} VTColor;
#define VT_COLORS 8

/* mgliche Zeichenstze: Text, zusammenhngende Grafik,
   separierte Grafik */
typedef enum {TEXTFONT = 0, CONGRAPHFONT = 1, SEPGRAPHFONT = 2} VTFont;

/* mgliche Schriftgren */
typedef enum {NORMALSIZE, DOUBLESIZE} VTFontSize;

/* In einem Feld dieses Datentyps wird die decodierte Seite gespeichert.
   Fr jedes der 40 * 24 Videotext-Zeichen ist dann einzeln erkennbar,
   in welchem Zeichensatz es dargestellt werden mu, welche Farbe es hat,
   ob es blinkt, ...
   Dies erleichtert die originalgetreue Darstellung der Seite */
typedef struct
{
  VTColor       bgndColor;  /* Hintergrundfarbe */
  VTColor       charColor;  /* Zeichenfarbe */
  VTFont        font;       /* Zeichentyp */
  VTFontSize    fontSize;   /* Zeichenhhe */
  BOOL          flashing;   /* blinkend */
  BOOL          hidden;     /* versteckt */
  unsigned char charCode;   /* Zeichenkode */
}VTChar;

